@@ -18,6 +18,7 @@ from email.mime import text
 import os
 import pkgutil
 import string
+from urllib import parse as urlparse
 
 from neutronclient.common import exceptions as q_exceptions
 from novaclient import api_versions
@@ -27,8 +28,6 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import netutils
-import six
-from six.moves.urllib import parse as urlparse
 import tenacity
 
 from heat.common import exception
@@ -82,11 +81,9 @@ class NovaClientPlugin(microversion_mixin.MicroversionMixin,
 
     def _get_args(self, version):
         endpoint_type = self._get_client_option(CLIENT_NAME, 'endpoint_type')
-        extensions = nc.discover_extensions(version)
 
         return {
             'session': self.context.keystone_session,
-            'extensions': extensions,
             'endpoint_type': endpoint_type,
             'service_type': self.COMPUTE,
             'region_name': self._get_region_name(),
@@ -227,7 +224,7 @@ class NovaClientPlugin(microversion_mixin.MicroversionMixin,
 
         """
         # not checking with is_uuid_like as most tests use strings e.g. '1234'
-        if isinstance(server, six.string_types):
+        if isinstance(server, str):
             server = self.fetch_server(server)
             if server is None:
                 return False
@@ -609,7 +606,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                 "nova server metadata needs to be a Map."))
 
         return dict((key, (value if isinstance(value,
-                                               six.string_types)
+                                               str)
                            else jsonutils.dumps(value))
                      ) for (key, value) in metadata.items())
 
@@ -677,7 +674,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                 except exceptions.UnsupportedConsoleType as ex:
                     url = ex.message
                 except Exception as e:
-                    url = _('Cannot get console url: %s') % six.text_type(e)
+                    url = _('Cannot get console url: %s') % str(e)
 
                 return url
 
@@ -696,9 +693,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                 volume_id=volume_id,
                 device=device)
         except Exception as ex:
-            if self.is_conflict(ex):
-                return False
-            elif self.is_client_exception(ex):
+            if self.is_client_exception(ex):
                 raise exception.Error(_(
                     "Failed to attach volume %(vol)s to server %(srv)s "
                     "- %(err)s") % {'vol': volume_id,
@@ -713,15 +708,12 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         try:
             self.client().volumes.delete_server_volume(server_id, attach_id)
         except Exception as ex:
-            if self.is_conflict(ex):
-                return False
-            elif not (self.is_not_found(ex)
-                      or self.is_bad_request(ex)):
+            if not (self.is_not_found(ex)
+                    or self.is_bad_request(ex)):
                 raise exception.Error(
                     _("Could not detach attachment %(att)s "
                       "from server %(srv)s.") % {'srv': server_id,
                                                  'att': attach_id})
-        return True
 
     def check_detach_volume_complete(self, server_id, attach_id):
         """Check that nova server lost attachment.
@@ -843,15 +835,6 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                     return True
         return False
 
-    @os_client.MEMOIZE_EXTENSIONS
-    def _list_extensions(self):
-        extensions = self.client().list_extensions.show_all()
-        return set(extension.alias for extension in extensions)
-
-    def has_extension(self, alias):
-        """Check if specific extension is present."""
-        return alias in self._list_extensions()
-
 
 class NovaBaseConstraint(constraints.BaseCustomConstraint):
 
