@@ -13,7 +13,6 @@
 
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
-import six
 
 from heat.common import exception
 from heat.common.i18n import _
@@ -206,7 +205,8 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
         ),
         STATUS: attributes.Schema(
             _('The current status of the volume.'),
-            type=attributes.Schema.STRING
+            type=attributes.Schema.STRING,
+            cache_mode=attributes.Schema.CACHE_NONE
         ),
         CREATED_AT: attributes.Schema(
             _('The timestamp indicating volume creation.'),
@@ -309,7 +309,7 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
         cinder = self.client()
         vol = cinder.volumes.get(self.resource_id)
         if name == self.METADATA_ATTR:
-            return six.text_type(jsonutils.dumps(vol.metadata))
+            return str(jsonutils.dumps(vol.metadata))
         elif name == self.METADATA_VALUES_ATTR:
             return vol.metadata
         if name == self.DISPLAY_NAME_ATTR:
@@ -318,7 +318,7 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             return vol.description
         elif name == self.ATTACHMENTS_LIST:
             return vol.attachments
-        return six.text_type(getattr(vol, name))
+        return str(getattr(vol, name))
 
     def check_create_complete(self, vol_id):
         complete = super(CinderVolume, self).check_create_complete(vol_id)
@@ -354,7 +354,7 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             if self.client_plugin().is_client_exception(ex):
                 raise exception.Error(_(
                     "Failed to extend volume %(vol)s - %(err)s") % {
-                        'vol': self.resource_id, 'err': six.text_type(ex)})
+                        'vol': self.resource_id, 'err': str(ex)})
             else:
                 raise
         return True
@@ -508,9 +508,17 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
 
     def _detach_volume_to_complete(self, prg_detach):
         if not prg_detach.called:
-            prg_detach.called = self.client_plugin(
-                'nova').detach_volume(prg_detach.srv_id,
-                                      prg_detach.attach_id)
+            # Waiting OS-EXT-STS:task_state in server to become available for
+            # detach
+            task_state = self.client_plugin('nova').fetch_server_attr(
+                prg_detach.srv_id, 'OS-EXT-STS:task_state')
+            # Wait till out of any resize steps (including resize_finish)
+            if task_state is not None and 'resize' in task_state:
+                prg_detach.called = False
+            else:
+                self.client_plugin('nova').detach_volume(prg_detach.srv_id,
+                                                         prg_detach.attach_id)
+                prg_detach.called = True
             return False
         if not prg_detach.cinder_complete:
             prg_detach.cinder_complete = self.client_plugin(
@@ -525,8 +533,16 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
 
     def _attach_volume_to_complete(self, prg_attach):
         if not prg_attach.called:
-            prg_attach.called = self.client_plugin('nova').attach_volume(
-                prg_attach.srv_id, prg_attach.vol_id, prg_attach.device)
+            # Waiting OS-EXT-STS:task_state in server to become available for
+            # attach
+            task_state = self.client_plugin('nova').fetch_server_attr(
+                prg_attach.srv_id, 'OS-EXT-STS:task_state')
+            # Wait till out of any resize steps (including resize_finish)
+            if task_state is not None and 'resize' in task_state:
+                prg_attach.called = False
+            else:
+                prg_attach.called = self.client_plugin('nova').attach_volume(
+                    prg_attach.srv_id, prg_attach.vol_id, prg_attach.device)
             return False
         if not prg_attach.complete:
             prg_attach.complete = self.client_plugin(
@@ -752,8 +768,17 @@ class CinderVolumeAttachment(vb.BaseVolumeAttachment):
             prg_detach = progress.VolumeDetachProgress(
                 server_id, volume_id, self.resource_id)
 
-            prg_detach.called = self.client_plugin(
-                'nova').detach_volume(server_id, self.resource_id)
+            # Waiting OS-EXT-STS:task_state in server to become available for
+            # detach
+            server = self.client_plugin('nova').fetch_server(server_id)
+            task_state = getattr(server, 'OS-EXT-STS:task_state', None)
+            # Wait till out of any resize steps (including resize_finish)
+            if task_state is not None and 'resize' in task_state:
+                prg_detach.called = False
+            else:
+                self.client_plugin('nova').detach_volume(server_id,
+                                                         self.resource_id)
+                prg_detach.called = True
 
             if self.VOLUME_ID in prop_diff:
                 volume_id = prop_diff.get(self.VOLUME_ID)
@@ -786,8 +811,16 @@ class CinderVolumeAttachment(vb.BaseVolumeAttachment):
                                                      self.resource_id)
             return False
         if not prg_attach.called:
-            prg_attach.called = self.client_plugin('nova').attach_volume(
-                prg_attach.srv_id, prg_attach.vol_id, prg_attach.device)
+            # Waiting OS-EXT-STS:task_state in server to become available for
+            # attach
+            server = self.client_plugin('nova').fetch_server(prg_attach.srv_id)
+            task_state = getattr(server, 'OS-EXT-STS:task_state', None)
+            # Wait till out of any resize steps (including resize_finish)
+            if task_state is not None and 'resize' in task_state:
+                prg_attach.called = False
+            else:
+                prg_attach.called = self.client_plugin('nova').attach_volume(
+                    prg_attach.srv_id, prg_attach.vol_id, prg_attach.device)
             return False
         if not prg_attach.complete:
             prg_attach.complete = self.client_plugin(
